/*
 * Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CohenVillemot.hh"
#include "Interpolator3D.hh"

class CohenVillemotVFI : public CohenVillemot
{
private:
  double *grid_a, *grid_a1;
  Interpolator3D VGNint, VGCint, a1_dr_normal_int,
    a1_dr_bonus_possible_int;

  const double maximization_tol;
  
  struct VG_params
  {
    double a;
    double y;
    double z;
    double *quad_points_y1, *quad_points_z1;
    const CohenVillemotVFI *CV;
  };

  static double VGN_objective(double a1, void *params);
  static double VGC_objective(double a1, void *params);

public:
  CohenVillemotVFI(double gam_arg, double beta_arg,
                   double lambda_arg, double delta_arg,
                   double p_arg, double q_arg, double mu_z_arg,
                   double recovery_arg, double convergence_tol_arg,
                   double min_a_arg);
  CohenVillemotVFI(const CohenVillemotVFI &arg);
  virtual ~CohenVillemotVFI();

  virtual double a1_dr(double a, double y, double z, bool bonus_possible) const;

  virtual void compute() throw (ConvergenceError);

  virtual double VGN(double a, double y, double z) const { return VGNint.get(a, y, z); };
  virtual double VGC(double a, double y, double z) const { return VGCint.get(a, y, z); };
};


