### Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
###
### This program is free software: you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation, either version 3 of the License, or
### (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program.  If not, see <http://www.gnu.org/licenses/>.

source("wrapper.R")

m <- new.ag.model(model = 2, gam = 2, beta = 0.8, lambda = 0.1, delta = 0.02,
                  convergence.tol = 1e-6, sol.method = "vfi")

min.a <- -0.3
max.a <- 0

p <- ag.get.params(m)

y.low <- p$mu.y - 6*p$sigma.g/sqrt(1-p$rho.g^2)
y.high <- p$mu.y + 6*p$sigma.g/sqrt(1-p$rho.g^2)
y.mean <- p$mu.y


x <- seq(min.a, max.a, by = 0.01)
z.low <- numeric()
z.high <- numeric()
z.mean <- numeric()

for (a in x)
  {
    z.low <- c(z.low, ag.a1.dr(m, a, y.low))
    z.high <- c(z.high, ag.a1.dr(m, a, y.high))
    z.mean <- c(z.mean, ag.a1.dr(m, a, y.mean))
  }

pdf(file = "choice-function.pdf", width = 6, height = 6, pointsize = 10)
plot(rev(-x), rev(-z.mean), type = "l", lty = 1, ylab = "D'", xlab = "D", ylim = c(0, 0.3))
lines(rev(-x), rev(-z.low), type = "l", lty = 2)
lines(rev(-x), rev(-z.high), type = "l", lty = 3)
legend("topleft", legend = c("Q = mean", "Q = mean - 6 sd", "Q = mean + 6 sd"), lty = 1:3)
dev.off()
