### Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
###
### This program is free software: you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation, either version 3 of the License, or
### (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program.  If not, see <http://www.gnu.org/licenses/>.

dyn.load('debt_models.so')

### Cohen-Villemot

new.cv.model <- function(gam = 2, beta = 0.95, lambda = 0.1, delta = 0.02, p = 0.015,
                         q = 0.05, mu.z = 0.01, recovery = -0.25*4,
                         sol.method = "2egm",
                         convergence.tol = ifelse(sol.method == "2egm", 1e-3, 2e-2),
                         min.a = ifelse(sol.method == "2egm", -2.5, -1.8),
                         debt.ceiling = FALSE, num.threads = 0)
  .Call('new_cv_model', gam, beta, lambda, delta, p, q, mu.z, recovery, convergence.tol, min.a,
        debt.ceiling, sol.method, as.integer(num.threads))

cv.statistics <- function(model, nsims = 500)
  .Call('cv_statistics', model, nsims)

cv.gdp <- function(model, y, z)
  .Call('cv_gdp', model, y, z)

cv.def <- function(model, a, y, z, bonus.possible = FALSE)
  .Call('cv_def', model, a, y, z, bonus.possible)

cv.a1.dr <- function(model, a, y, z, bonus.possible = FALSE)
  .Call('cv_a1_dr', model, a, y, z, bonus.possible)

cv.spread <- function(model, y, z, bonus.possible = FALSE, a1)
  .Call('cv_spread', model, y, z, bonus.possible, a1)

cv.crisis.prob <- function(model, y, z, bonus.possible = FALSE, a1)
  .Call('cv_crisis_prob', model, y, z, bonus.possible, a1)

cv.get.params <- function(model)
  .Call('cv_get_params', model)

cv.V <- function(model, a, y, z, bonus.possible)
  .Call('cv_V', model, a, y, z, bonus.possible)

cv.VGN <- function(model, a, y, z)
  .Call('cv_VGN', model, a, y, z)

cv.VGC <- function(model, a, y, z)
  .Call('cv_VGC', model, a, y, z)

cv.VBC <- function(model, y, z)
  .Call('cv_VBC', model, y, z)

cv.VBN <- function(model, y, z)
  .Call('cv_VBN', model, y, z)

cv.repayment.probability <- function(model, y, z1, a1, bonus.possible1, deriv.a1 = NULL)
  .Call('cv_repayment_probability', model, y, z1, a1, bonus.possible1, deriv.a1)

cv.price <- function(model, y, z, bonus.possible, a1, deriv.a1 = NULL)
  .Call('cv_price', model, y, z, bonus.possible, a1, deriv.a1)

cv.cons <- function(model, a, y, z, bonus.possible, d)
  .Call('cv_cons', model, a, y, z, bonus.possible, as.integer(d))

cv.simulate <- function(model, a, y, z, d, bonus.possible, seed = 0)
  .Call('cv_simulate', model, a, y, z, d, bonus.possible, as.integer(seed))

### Aguiar-Gopinath

new.ag.model <- function(model = 2, gam = 2, beta = 0.8, lambda = 0.1, delta = 0.02,
                         convergence.tol = 1e-6, ngrid.a = 30, ngrid.y = 15, sol.method = "2egm",
                         num.threads = 0)
  .Call('new_ag_model', model, gam, beta, lambda, delta, convergence.tol, ngrid.a, ngrid.y, sol.method, as.integer(num.threads))

ag.statistics <- function(model, nsims = 500)
  .Call('ag_statistics', model, nsims)

ag.gdp <- function(model, y)
  .Call('ag_gdp', model, y)

ag.def <- function(model, a, y)
  .Call('ag_def', model, a, y)

ag.a1.dr <- function(model, a, y)
  .Call('ag_a1_dr', model, a, y)

ag.get.params <- function(model)
  .Call('ag_get_params', model)

ag.has.converged <- function(model)
  .Call('ag_has_converged', model)

ag.V <- function(model, a, y)
  .Call('ag_V', model, a, y)

ag.q <- function(model, y, a1, deriv.a1 = NULL)
  .Call('ag_q', model, y, a1, deriv.a1)

ag.simulate <- function(model, y, a, d, seed = 0)
  .Call('ag_simulate', model, y, a, d, as.integer(seed))

ag.cons <- function(model, a, y, d)
  .Call('ag_cons', model, a, y, as.integer(d))
